#!/usr/bin/perl -w
use Carp qw| cluck :DEFAULT |;

#this scrip will either substitute all residues of a protein
# by all other possible residues or by another specified residue
# or use the current mutant file to create all possible mutations at given positions
#command line
# 1 -- mutant file or directory containing mutant file
# 2 -- substitution by : 'all', 'X', or '2' (all and X are for single file, 2 is for all mutants in directory)
# 3 -- add something to extract synonymous subs as well
if ($ARGV[0] =~ /\/$/){
	opendir (IN, $ARGV[0]);
	@files = readdir IN;
	closedir IN;
	$sub = $ARGV[1];
	foreach $prot_file (@files){
		
		if ($prot_file =~ /\.muts/){
			$prot_file = $ARGV[0]."/$prot_file";
			&runProg();
		}
	}
}
else{	
	$prot_file = $ARGV[0];
	$sub = $ARGV[1];
	&runProg();
}

exit(0);

sub runProg{
	if( $main::dbg ){ cluck( "processing $prot_file"); }
	#if we are substituting by all possible amino acids only the mutations that are already there
	if ($sub =~ /\d/){
		$mut_file = `more $prot_file`;
		open (OUT, ">$prot_file") || die "Can't open $prot_file\n";
		@amino = ('A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y');
		@mut = split (/\n/, $mut_file);
		$pos = " ";
		foreach $line (@mut){
			$line =~ s/([A-Z])(\d+)[ACDEFGHIKLMNPQRSTVWY]\s+([^\s]+)//g;
			$org = $1;
			$dig = $2;
			$score = $3;
			if ($pos =~ / $dig /){
				next;
			}
			else{
				$pos .= "$dig ";
			}
			foreach $aa (@amino){
				#if ($aa eq $org){
				#	next;
				#}
				#else{
					print OUT "$org$dig$aa $score\n";
				#}
			}
		}
		close OUT;
		return;
	}
	#else
	
	$name = $prot_file;
	$name =~ s/(.+)\/([^\/]+)\.(muts|fasta)//;
	$dir = $1;
	$name = $2;
	$name =~ s/\_query//;
	$full = $prot_file;
	$full =~ s/\.muts/\.fasta/;
	if( $main::dbg ){ cluck("Name of the protein is $name. Directory is $dir");}
	open (FILE, $full) || die "Can't open $full for reading";
	@file = <FILE>;
	$file = "@file";
	close FILE;
	if (!$file){ confess( "$prot_file doesn't exist");}
	$file =~ s/\>.*\n//;
	$file =~ tr/a-z/A-Z/;
	$file =~ s/[^A-Z]//g;
	$file =~ s/\n//g;
	$file =~ s/(.)/$1 /g;
	@seq = split (/ /, $file);
	if( $main::dbg ){ cluck( "@seq" ); }
	if (!($dir)){
		$dir =`pwd`;
		$dir =~ s/\n//;
	}
	if ($ARGV[2]){
		if( $main::dbg ){ cluck( "$dir/$name.Allmuts"); }
		open (OUT, ">", "$dir/$name.Allmuts") || die "Can't open $dir/$name.Allmuts";
	}
	else{
		open (OUT, ">$dir/$name.muts") || die "Can't open $dir/$name.muts";
	}
	if ($sub =~ /all/){
		@amino = ('A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y');
		$count = 1;
		foreach $aa (@seq){
			if (!($aa =~ /B|J|O|U|X|Z/)){
				foreach $ab (@amino){
					if ((!($aa eq $ab)) or ($ARGV[2])){
						print OUT $aa.$count.$ab."\n";
					}
				}
			}
			$count++;
		}
	}
	elsif ($sub =~ /^[A-Z]$/){
		$count = 1;
		foreach $aa (@seq){
			if (!($aa =~ /B|J|O|U|X|Z/)){
				if (!($aa eq $sub)){
					print OUT "$aa$count$sub\n";
				}
			}
			$count++;
		}
	}
	else{
		if( $main::dbg ){ cluck("This is not a possible substitution: $sub"); }
	}	
	close OUT;
	return;
}
