package runAll;
use strict;
use Carp qw(cluck :DEFAULT);
use File::chdir;
use Scalar::Util qw||;
use Data::Dumper;
#this is a sub collection for extracting 
#needed information for each sequence
#$pd = $ENV{PROF};

sub runAll{
	my( $__workdir, $__jobname, $__job_fasta_file, $debug, $__tolerate_psic_failure, $__tolerate_sift_failure, $__blastpgp_processors,
        $__pfamdata, $__uniref, $__swiss, $__uniref90, $mode, $__forcecachestore
    ) = @_;
	$__blastpgp_processors ||= 1;
	$mode ||="optimized";

    my $blastpgp_seg_filter = $main::config->val('snapfun', 'blastpgp_segfilter') || 'F'; if( $blastpgp_seg_filter !~ /^([FT])$/o ){ confess("value '$blastpgp_seg_filter' is not valid for blastpgp_seg_filter"); } $blastpgp_seg_filter = $1;
    my $librg_utils_perl = glob( $main::config->val('snapfun', 'librg_utils_perl') ) || '/usr/share/librg-utils-perl';
    my $snapfun_utildir = glob( $main::config->val('snapfun', 'snapfun_utildir') );
	my $psic_matrix = glob( $main::config->val('snapfun', 'psic_matrix') );
	my $runpsic = glob( $main::config->val('snapfun', 'runpsic') );
	my $hmm2pfam = glob( $main::config->val('snapfun', 'hmmpfam') );
	#my $sift = glob( $main::config->val('snapfun', 'sift') ) || 'info_on_seqs';
	my $pp_PROFNUMRESMIN=17; #this value comes from the MakefilePP.mk - Predict Protein
	if( $__forcecachestore && $mode ne "predictprotein" ){ die "Error: --force-cache-store is incompatible with $mode, please refer to the man page.\n"; } #--force-cache-store implies --mode=predictprotein
	
	#mhecht: different commands depending on the version of SNAP
	my %cmd;
	if ($mode eq "published") #The original version as published by Yana Bromberg and Burkhard Rost NAR (2007)
	{
		$cmd{blastbig}=qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__uniref' -o '$__workdir/$__jobname/$__jobname.blast' -Q '$__workdir/$__jobname/$__jobname.asci' -e 1 -j 7 -h 0.001 -b 3000|.( $debug ? '' : ' >/dev/null 2>&1' );
		$cmd{blast2saf}=qq|$librg_utils_perl/blast2saf.pl $__workdir/$__jobname/$__jobname.blast fasta=$__workdir/$__jobname/$__job_fasta_file saf=$__workdir/$__jobname/$__jobname.saf|;
		$cmd{saf2filtersaf}=qq|$librg_utils_perl/safFilterRed.pl $__workdir/$__jobname/$__jobname.saf red=85 fileOut=$__workdir/$__jobname/$__jobname.safFil|;
		$cmd{filtersaf2hssp}=qq|$librg_utils_perl/copf.pl $__workdir/$__jobname/$__jobname.safFil formatIn=saf formatOut=hssp fileOut=$__workdir/$__jobname/$__jobname.hssp exeConvertSeq=convert_seq|;
		$cmd{hssp2filterhssp}=qq|$librg_utils_perl/hssp_filter.pl $__workdir/$__jobname/$__jobname.hssp red=80 dirOut=$__workdir/$__jobname exeFilterHssp=filter_hssp|;
		$cmd{filterhssp2rdbProf}=qq|prof both nonice $__workdir/$__jobname/$__jobname-fil.hssp dirOut=$__workdir/$__jobname/|;
		$cmd{fasta2rdbProf}=qq|prof both nonice $__workdir/$__jobname/$__job_fasta_file dirOut=$__workdir/$__jobname/|;
		$cmd{pfam}=qq|$snapfun_utildir/run_hmmer.pl --blastpgp-processors $__blastpgp_processors -l $__workdir/$__jobname/list.$__jobname -dirSeq $__workdir/$__jobname -dirOut $__workdir/$__jobname --pfamdata $__pfamdata|.( $debug ? ' --debug' : () );
		$cmd{sift}=qq|sift_for_submitting_fasta_seq.csh '$__workdir/$__jobname/$__jobname\_query.fasta' '$__uniref90' '$__workdir/$__jobname/$__jobname.Allmuts' '$__blastpgp_processors' |.( $debug ? '' : ' >/dev/null 2>&1' );
		$cmd{psic}=qq|$runpsic --infile $__workdir/$__jobname/$__job_fasta_file --workDir $__workdir/$__jobname --run 0 --min-seqlen 50 --blastpgp-processors $__blastpgp_processors --blastdata_uniref $__uniref --blastpgp_seg_filter $blastpgp_seg_filter --psic_matrix $psic_matrix|;
		$cmd{profbval}=qq|profbval $__workdir/$__jobname/$__job_fasta_file $__workdir/$__jobname/$__jobname-fil.rdbProf $__workdir/$__jobname/$__jobname-fil.hssp $__workdir/$__jobname/$__jobname.profbval 1 snapfun|;
		$cmd{refilterhssp}=qq|$librg_utils_perl/hssp_filter.pl $__workdir/$__jobname/$__jobname.hssp red=60 dirOut=$__workdir/$__jobname exeFilterHssp=filter_hssp|;
		$cmd{blastswiss}=qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__swiss' -e 0.001 -o '$__workdir/$__jobname/$__jobname.blastswiss'|.( $debug ? '' : ' >/dev/null 2>&1' );	
	}
	elsif ($mode eq "optimized") #the commands for standalone and cached version are identical, this is the default case 
	{ 
		$cmd{blastbig}=qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__uniref90' -o '$__workdir/$__jobname/$__jobname.blast' -Q '$__workdir/$__jobname/$__jobname.asci' -e 1 -j 3 -h 0.001 -b 3000|.( $debug ? '' : ' >/dev/null 2>&1' );
		$cmd{blast2saf}=qq|$librg_utils_perl/blastpgp_to_saf.pl fileInBlast=$__workdir/$__jobname/$__jobname.blast fileInQuery=$__workdir/$__jobname/$__job_fasta_file fileOutRdb=$__workdir/$__jobname/$__jobname.blastPsiRdb fileOutSaf=$__workdir/$__jobname/$__jobname.saf red=100 maxAli=3000 tile=0 fileOutErr=$__workdir/$__jobname/$__jobname.blast2safErr|;
		$cmd{saf2filtersaf}=qq|cp $__workdir/$__jobname/$__jobname.saf $__workdir/$__jobname/$__jobname.safFil|;
		$cmd{filtersaf2hssp}=qq|$librg_utils_perl/copf.pl $__workdir/$__jobname/$__jobname.safFil formatIn=saf formatOut=hssp fileOut=$__workdir/$__jobname/$__jobname.hssp exeConvertSeq=convert_seq|;
		$cmd{hssp2filterhssp}=qq|$librg_utils_perl/hssp_filter.pl red=80 $__workdir/$__jobname/$__jobname.hssp fileOut=$__workdir/$__jobname/$__jobname-fil.hssp|;
		$cmd{refilterhssp}=qq|$librg_utils_perl/hssp_filter.pl red=60 $__workdir/$__jobname/$__jobname.hssp fileOut=$__workdir/$__jobname/$__jobname-fil.hssp|;
		$cmd{filterhssp2rdbProf}=qq|prof both $__workdir/$__jobname/$__jobname-fil.hssp fileRdb=$__workdir/$__jobname/$__jobname-fil.rdbProf numresMin=$pp_PROFNUMRESMIN nresPerLineAli=60 riSubSec=4 riSubAcc=3 riSubSym=.|;
		$cmd{fasta2rdbProf}=qq|prof both $__workdir/$__jobname/$__job_fasta_file fileRdb=$__workdir/$__jobname/$__jobname.rdbProf numresMin=$pp_PROFNUMRESMIN nresPerLineAli=60 riSubSec=4 riSubAcc=3 riSubSym=.|;
		$cmd{pfam}=qq|$hmm2pfam --cpu $__blastpgp_processors --acc --cut_ga $__pfamdata $__workdir/$__jobname/$__job_fasta_file > $__workdir/$__jobname/$__jobname.pfam|;
		$cmd{psic}=qq|$runpsic --save-clustaln '$__workdir/$__jobname/$__jobname.clustal' --use-blastfile '$__workdir/$__jobname/$__jobname.blast' --infile '$__workdir/$__jobname/$__job_fasta_file' --quiet --min-seqlen '$pp_PROFNUMRESMIN'  --blastpgp-processors '$__blastpgp_processors' --blastdata_uniref '$__uniref' --blastpgp_seg_filter '$blastpgp_seg_filter' --psic_matrix '$psic_matrix' --psicfile '$__workdir/$__jobname/$__jobname.out'|;
		$cmd{blastswiss}=qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__swiss' -e 0.001 -o '$__workdir/$__jobname/$__jobname.blastswiss'|.( $debug ? '' : ' >/dev/null 2>&1' );			
		$cmd{profbval}=qq|profbval $__workdir/$__jobname/$__job_fasta_file $__workdir/$__jobname/$__jobname-fil.rdbProf $__workdir/$__jobname/$__jobname-fil.hssp $__workdir/$__jobname/$__jobname.profbval 1 snapfun|;		
		#This sift call reuses the clustal file that is produced from psic; This skips the blasting and selecting step within sift but results in a slight performance loss
		$cmd{sift}=qq|info_on_seqs '$__workdir/$__jobname/$__jobname.clustal' '$__workdir/$__jobname/$__jobname.muts' '$__workdir/$__jobname/$__jobname\_query.SIFTprediction'|.( $debug ? '' : ' >/dev/null 2>&1' );
		#$cmd{sift}=qq|sift_for_submitting_fasta_seq.csh '$__workdir/$__jobname/$__jobname\_query.fasta' '$__uniref90' '$__workdir/$__jobname/$__jobname.Allmuts' '$__blastpgp_processors' |.( $debug ? '' : ' >/dev/null 2>&1' );
		$cmd{formatclustal}=qq|$snapfun_utildir/ungap_clustal.pl $__workdir/$__jobname/$__jobname.clustal|;
	}	
	elsif ($mode eq "predictprotein") #using the predict protein cache to re-use results
	{ 
			#Run Predict Protein to produce all the missing files we need from the cache; see predict protein manual for details
			# lkajan: Max,
			# there is a good reason for using ARRAYs for system and not strings: it is much safer.
			# That is because the ARRAY elements are not processed by the shell and so no shell redirects/escapes etc. get processed. By mistake, because here we do not want the shell to interfere. So it's better to make sure it can not happen - by using the ARRAY with system.
			# You will see that I use system with a string only where I /do/ want shell redirection - usually of stderr or stdout.
			# There are also such things like semicolon, pipe character, quotes and spaces in filenames that would totally confuse a system( <string> ), think about: my $file = "afile bfile | rm -rf ~ "; rm -rf "; system( "cp $file /tmp" ); OR my $file = "/tmp/laszlo's file"; system( "cp '$file' /tmp" ); - neither of these is going to do the copying.
			# In this particular case __workdir may come from the command line. So there may be spaces, quotes, '<>' and shell special characters. So your solution of 'cp's below does not work reliably.
			#
			# It would be a good idea to check workdir for a set of allowed characters (perhaps [[:alnum:]/,." _-]) at the entry of this subroutine.  Also there should be a check where it comes in from the CMD line and an error thrown if there are forbidden characters.
			#
			# lkajan 20111001: please update the man page (for snapfun) with the list of files taken from the cache when you change it
			my @run_pp = ( 'predictprotein', '--seqfile', "$__workdir/$__jobname/$__job_fasta_file",
				"--target=query.blastPsiMat",
				"--target=query.blastPsiOutTmp",
				"--target=query.clustalngz",
				"--target=query.hmm2pfam",
				"--target=query.hsspPsiFil",
				"--target=query.prof1Rdb",
				"--target=query.profb4snap",
				"--target=query.profRdb",
				"--target=query.psic",
				"--blast-processors", $__blastpgp_processors,
				( $__forcecachestore ? '--force-cache-store' : '' )
		 	);
			if ($debug) { push @run_pp, '--debug'; cluck("@run_pp");}
			system(@run_pp) && confess("@run_pp failed: ".($?>>8) );	
				
			#additionally if we don't have all the files we need in our --workdir from a previous run: look up in the cache
			if(!(-e "$__workdir/$__jobname/$__jobname.out") or !(-e "$__workdir/$__jobname/$__jobname-fil.rdbProf") or !(-e "$__workdir/$__jobname/$__jobname.asci") or !(-e "$__workdir/$__jobname/$__jobname-fil.hssp") or !(-e "$__workdir/$__jobname/$__jobname.blast"))
			{
				# lkajan 20110830: so you are only interested in the directory of the cache slot: use the -d argument
				open( my $ph, '-|', 'ppc_fetch', '--seqfile', "$__workdir/$__jobname/$__job_fasta_file", '-d' ) || confess( "failed to open pipe: $!" );
				my( $cache_slot_dir ) = <$ph>; if( !$cache_slot_dir ){ confess("no results in cache for sequence '$__workdir/$__jobname/$__job_fasta_file'"); }
				if( !close( $ph ) ){ if( $! == 0 && ( $? >> 8 ) == 254 ){ confess("no results in cache for sequence '$__workdir/$__jobname/$__job_fasta_file'"); } else { confess( "failed to close pipe: $!" ); } }
				chomp( $cache_slot_dir );
				my $cachedblast =	$cache_slot_dir."/query.blastPsiOutTmp";
				my $cachedascii =	$cache_slot_dir."/query.blastPsiMat";
				my $cachedpsic =	$cache_slot_dir."/query.psic";
				my $cachedpfam =	$cache_slot_dir."/query.hmm2pfam";
				my $cachedprof =	$cache_slot_dir."/query.prof1Rdb";
				my $cachedrdbprof =	$cache_slot_dir."/query.profRdb";
				my $cachedhssp =	$cache_slot_dir."/query.hsspPsiFil";
				my $cachedclustal =	$cache_slot_dir."/query.clustalngz"; 
				my $cachedprofbval=	$cache_slot_dir."/query.profb4snap"; 
				

				#copy files to workdir; if successful, the corresponding extraction step will be skipped 
				{my @cmd = ( "cp", $cachedblast,    "$__workdir/$__jobname/$__jobname.blast" );  		cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))}
				{my @cmd = ( "cp", $cachedascii,    "$__workdir/$__jobname/$__jobname.asci" ); 			cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))} 	
				{my @cmd = ( "cp", $cachedpfam,     "$__workdir/$__jobname/$__jobname.pfam" ); 			cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))}
				{my @cmd = ( "cp", $cachedprof,     "$__workdir/$__jobname/$__jobname.rdbProf" ); 		cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))}
				{my @cmd = ( "cp", $cachedhssp,     "$__workdir/$__jobname/$__jobname-fil.hssp" ); 		cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))}
				{my @cmd = ( "cp", $cachedrdbprof,  "$__workdir/$__jobname/$__jobname-fil.rdbProf" ); 	cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))}
				{my @cmd = ( "cp", $cachedprofbval, "$__workdir/$__jobname/$__jobname.profbval" ); 		cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))}
				{my @cmd = ( "cp", $cachedclustal,  "$__workdir/$__jobname/$__jobname.clustal.gz" ); 	cluck("@cmd") if ($debug); system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))}
				
				#There may be cases where no blast hits were found - catch these:
				chomp(my $checkpsic=`head -1 $cachedpsic`);
				if ($checkpsic eq "blastpgp: No hits found") {
					if (!($__tolerate_psic_failure)){
						confess( "'Error - $checkpsic for PSIC'" )
					}
					else {
						my @cmd = ("touch", "$__workdir/$__jobname/$__jobname.out" );
						cluck("@cmd") if ($debug); 
						system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ))
					}
				}
				else {
					{
					my @cmd = ("cp", $cachedpsic, "$__workdir/$__jobname/$__jobname.out" ); 
					cluck("@cmd") if ($debug); 
					system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ));
					}
					{
					my @cmd = ("gunzip", "$__workdir/$__jobname/$__jobname.clustal.gz" ); 
					cluck("@cmd") if ($debug); 
					system(@cmd) && confess("failed to '@cmd' ".( $?>>8 ));
					}
				}
		}
		$cmd{blastswiss}=qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__swiss' -e 0.001 -o '$__workdir/$__jobname/$__jobname.blastswiss'|.( $debug ? '' : ' >/dev/null 2>&1' );			
		$cmd{sift}=qq|info_on_seqs '$__workdir/$__jobname/$__jobname.clustal' '$__workdir/$__jobname/$__jobname.muts' '$__workdir/$__jobname/$__jobname\_query.SIFTprediction'|.( $debug ? '' : ' >/dev/null 2>&1' );
		$cmd{formatclustal}=qq|$snapfun_utildir/ungap_clustal.pl $__workdir/$__jobname/$__jobname.clustal|;
		
	}
	else {die"Error: Invalid mode: $mode\n\tPlease check the valid modes on the man page or examples in the config\n"}
		

	
	if( $debug ){ cluck( Dumper( \%cmd ) ); }	
	
	#if we (still) don't have the secondary structure prediction from alignment -- extract it
	if( !(-e "$__workdir/$__jobname/$__jobname-fil.rdbProf") or !(-e "$__workdir/$__jobname/$__jobname.asci") or !(-e "$__workdir/$__jobname/$__jobname.blast") )
	{
	
		#first the .saf and .asci files
		if (!(-e "$__workdir/$__jobname/$__jobname.blast") or !(-e "$__workdir/$__jobname/$__jobname.asci")){
			
			#my $cmd = qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__uniref' -o '$__workdir/$__jobname/$__jobname.blast' -Q '$__workdir/$__jobname/$__jobname.asci' -e 1 -j 7 -h 0.001 -b 3000|.( $debug ? '' : ' >/dev/null 2>&1' );
			#my $cmd = qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__uniref90' -o '$__workdir/$__jobname/$__jobname.blast' -Q '$__workdir/$__jobname/$__jobname.asci' -e 1 -j 3 -h 0.001 -b 3000|.( $debug ? '' : ' >/dev/null 2>&1' );
		    cluck( $cmd{blastbig} ) if ($debug);
			system( $cmd{blastbig} ) && confess( "$cmd{blastbig} failed: ".($?>>8) );

			if (!(-e "$__workdir/$__jobname/$__jobname.blast")){
				my $error = "Couldn't run blastpgp to get $__jobname.blast file\n";
				return $error;
			}
		}
		elsif( $debug ){ warn("reusing '$__workdir/$__jobname/$__jobname.blast' and '$__workdir/$__jobname/$__jobname.asci'"); }

		#convert blast to saf
		if (!(-e "$__workdir/$__jobname/$__jobname.saf")){
		    #mhecht: using cache variant- in addition to standard creates $__jobname.blastPsiRdb and $__jobname.blast2safErr ; for cache storing we will need to: cp $__workdir/$__jobname/$__jobname.saf query.safBlastPsi
		    #my @cmd = ( "$librg_utils_perl/blast2saf.pl", "$__workdir/$__jobname/$__jobname.blast", "fasta=$__workdir/$__jobname/$__job_fasta_file", "saf=$__workdir/$__jobname/$__jobname.saf" );
		    #my @cmd = ( "$librg_utils_perl/blastpgp_to_saf.pl", "fileInBlast=$__workdir/$__jobname/$__jobname.blast", "fileInQuery==$__workdir/$__jobname/$__job_fasta_file", "fileOutRdb=$__workdir/$__jobname/$__jobname.blastPsiRdb", "fileOutSaf=$__workdir/$__jobname/$__jobname.saf", "red=100", "maxAli=3000", "tile=0", "fileOutErr=$__workdir/$__jobname/$__jobname.blast2safErr" );
		    cluck($cmd{blast2saf}) if ($debug);
		    system( $cmd{blast2saf} ) && confess( "$cmd{blast2saf} failed: ".($?>>8) );
		    if (!(-e "$__workdir/$__jobname/$__jobname.saf")){
				my $error = "Couldn't run blastpgp_to_saf to get $__jobname.saf file\n";
				return $error;
			}
		}
		
		
		#then filter the .saf files to
		if (!(-e "$__workdir/$__jobname/$__jobname.safFil")){
		# 	mhecht: we will skip this step here since the hssp file will be filtered
		#	my @cmd = ("$librg_utils_perl/safFilterRed.pl", "$__workdir/$__jobname/$__jobname.saf", "red=85", "fileOut=$__workdir/$__jobname/$__jobname.safFil");
		#   my @cmd = ("cp", "$__workdir/$__jobname/$__jobname.saf", "$__workdir/$__jobname/$__jobname.safFil");
		    cluck($cmd{saf2filtersaf}) if ($debug);
			system( $cmd{saf2filtersaf} ) && confess( "$cmd{saf2filtersaf} failed: ".($?>>8) );
			if (!(-e "$__workdir/$__jobname/$__jobname.safFil")){			
				my $error = "Couldn't run safFilterRed.pl to get $__jobname.safFil file\n";
				return $error;
			}
		}
		
		#get the .hssp files
		if (!(-e "$__workdir/$__jobname/$__jobname.hssp")){
		#make sure that sequence names in the .safFil are under 14 letters long
                        open (IN, "$__workdir/$__jobname/$__jobname.safFil");
                        open (OUT, ">$__workdir/$__jobname/$__jobname.safFil.tmp");
                        foreach my $line (<IN>){
                                if (!($line =~ /\#/)){
                                        $line =~ s/.+\|//;
                                        $line =~ s/.+\_query/\_Query\_/;
										$line =~ s/([^ ]+)//;
										my $qn = $1;
										if (length($qn) > 14){
											$qn =~ s/(.{14}).+/$1/;
										}
										$line = $qn.$line;
										print OUT $line;
                                }
                                else{
                                        print OUT $line;
                                }
                        }
                        close IN;
                        close OUT;
			{
				my @cmd = ("mv", "$__workdir/$__jobname/$__jobname.safFil.tmp", "$__workdir/$__jobname/$__jobname.safFil" );
				system( @cmd ) && confess( "'@cmd' failed: ".($?>>8) );
			}
			{
				#mhecht slightly edited call to match the pp cache - this creates .hssp80 file
				#my @cmd = ("$librg_utils_perl/copf.pl", "$__workdir/$__jobname/$__jobname.safFil", "$__workdir/$__jobname/$__jobname.hssp", "exeConvertSeq=convert_seq" );
				#my @cmd = ("$librg_utils_perl/copf.pl", "$__workdir/$__jobname/$__jobname.safFil", "formatIn=saf", "formatOut=hssp", "fileOut=$__workdir/$__jobname/$__jobname.hssp", "exeConvertSeq=convert_seq" );
				cluck($cmd{filtersaf2hssp}) if ($debug);
				system( $cmd{filtersaf2hssp} ) && confess("$cmd{filtersaf2hssp} failed: ".($?>>8));
			}
			if (!(-e "$__workdir/$__jobname/$__jobname.hssp")){
				my $error = "Couldn't run copf.pl to get $__jobname.hssp file ($librg_utils_perl/copf.pl $__workdir/$__jobname/$__jobname.safFil $__workdir/$__jobname/$__jobname.hssp exeConvertSeq=convert_seq)\n";
				return $error;
			}
		}
		#filter the hssp files as well
		if (!(-e "$__workdir/$__jobname/$__jobname-fil.hssp"))
		{
			#mhecht: sligthly edited; for storing we will need to cp $__jobname-fil.hssp query.hsspPsiFil
			#my @cmd = ("$librg_utils_perl/hssp_filter.pl", "$__workdir/$__jobname/$__jobname.hssp", "red=80", "dirOut=$__workdir/$__jobname", "exeFilterHssp=filter_hssp");
			#my @cmd = ("$librg_utils_perl/hssp_filter.pl", "red=80", "$__workdir/$__jobname/$__jobname.hssp", "fileOut=$__workdir/$__jobname/$__jobname-fil.hssp");
		    cluck($cmd{hssp2filterhssp}) if ($debug);
			system( $cmd{hssp2filterhssp} ) && confess( "$cmd{hssp2filterhssp} failed: ".($?>>8) );
			if (!(-e "$__workdir/$__jobname/$__jobname-fil.hssp")){
				my $error = "Couldn't run hssp_filter.pl to get $__jobname-fil.hssp file\n";
				return $error;
			}
		}
		#get the secondary structure (.rdbProf files)
		if (!(-e "$__workdir/$__jobname/$__jobname-fil.rdbProf")){
			#my @cmd = ("prof", "both", "nonice", "$__workdir/$__jobname/$__jobname-fil.hssp", "dirOut=$__workdir/$__jobname/" );
		    cluck($cmd{filterhssp2rdbProf}) if ($debug);
			system( $cmd{filterhssp2rdbProf} ) && confess( "$cmd{filterhssp2rdbProf} failed: ".($?>>8) );
			#if the filtered hssp file is too big -- reduce the sequence identity and re-run extraction
			if (!(-e "$__workdir/$__jobname/$__jobname-fil.rdbProf"))
			{
				{
					#my @cmd =("$librg_utils_perl/hssp_filter.pl", "$__workdir/$__jobname/$__jobname.hssp", "red=60", "dirOut=$__workdir/$__jobname", "exeFilterHssp=filter_hssp");
			    	cluck($cmd{refilterhssp}) if ($debug);
					system( $cmd{refilterhssp} ) && confess( "$cmd{refilterhssp} failed: ".($?>>8) );
				}
				{
					#my @cmd = ("prof", "both", "nonice", "$__workdir/$__jobname/$__jobname-fil.hssp", "dirOut=$__workdir/$__jobname/" );
					cluck($cmd{filterhssp2rdbProf}) if ($debug);
					system( $cmd{filterhssp2rdbProf} ) && confess( "$cmd{filterhssp2rdbProf} failed: ".($?>>8) );
				}
				#if still a problem -- fail
				if (!(-e "$__workdir/$__jobname/$__jobname-fil.rdbProf")){
					my $error = "Couldn't run prof.pl to get $__jobname-fil.rdbProf file\n";
					return $error;
				}
			}
		}
	}

	#get the secondary structure from sequence alone (no alignments)
	if (!(-e "$__workdir/$__jobname/$__jobname.rdbProf")){
		{
			#my @cmd = ("prof", "both", "nonice", "$__workdir/$__jobname/$__job_fasta_file", "dirOut=$__workdir/$__jobname/");
	    	cluck($cmd{fasta2rdbProf}) if ($debug);
			system( $cmd{fasta2rdbProf} ) && confess( "$cmd{fasta2rdbProf} failed: ".($?>>8) );
		}
		if ($mode eq 'published' ) {
			my @cmd = ("mv", "$__workdir/$__jobname/$__jobname\_query.rdbProf", "$__workdir/$__jobname/$__jobname.rdbProf");
			system( @cmd ) && confess( "'@cmd' failed: ".($?>>8) );
		}
		if (!(-e "$__workdir/$__jobname/$__jobname.rdbProf")){
			my $error = "Couldn't run prof.pl to get $__jobname.rdbProf file\n";
			return $error;
		}
	}

	#get the swiss-prot annotation if exists
	if (!(-e "$__workdir/$__jobname/$__jobname.blastswiss")){
		#my $cmd = qq|blastpgp -F $blastpgp_seg_filter -a $__blastpgp_processors -i '$__workdir/$__jobname/$__job_fasta_file' -d '$__swiss' -e 0.001 -o '$__workdir/$__jobname/$__jobname.blastswiss'|.( $debug ? '' : ' >/dev/null 2>&1' );
		cluck($cmd{blastswiss}) if ($debug);
		system( $cmd{blastswiss} ) && confess("$cmd{blastswiss} failed: ".($?>>8));

		if (!(-e "$__workdir/$__jobname/$__jobname.blastswiss")){
			my $error = "Couldn't run blastpgp to get $__jobname.blastswiss file\n $cmd{blastswiss}\n";
			return $error;
		}
	}

	#get the pfam domains	
	if (!(-e "$__workdir/$__jobname/$__jobname.pfam")){
		open (IN, ">$__workdir/$__jobname/list.$__jobname") || die "Can't open $__workdir/$__jobname/list.$__jobname";
		print IN "$__job_fasta_file";
		close IN;
		{
			#my @cmd = ( "$snapfun_utildir/run_hmmer.pl", '--blastpgp-processors', $__blastpgp_processors, '-l', "$__workdir/$__jobname/list.$__jobname", "-dirSeq", "$__workdir/$__jobname", "-dirOut", "$__workdir/$__jobname", '--pfamdata', $__pfamdata, ( $debug ? '--debug' : () ) );
			cluck($cmd{pfam}) if ($debug);  
			system( $cmd{pfam} ) && confess( "$cmd{pfam} failed: ".($?>>8) );
		}
		
		if ($mode eq 'published' ){
			
			my @cmd = ("mv", "$__workdir/$__jobname/$__jobname\_query.pfam", "$__workdir/$__jobname/$__jobname.pfam" );
			system( @cmd ) && confess( "'@cmd' failed: ".($?>>8) );
		}
		if (!(-e "$__workdir/$__jobname/$__jobname.pfam")){
			my $error = "Couldn't run run_hmmer.pl to get $__jobname.pfam file\n";
			return $error;
		}
		unlink( "$__workdir/$__jobname/list.$__jobname" );
	}
	#get the psic file
	if (!(-e "$__workdir/$__jobname/$__jobname.out")){

#		my @cmd = ( $runpsic, '--infile', "$__workdir/$__jobname/$__job_fasta_file", '--workDir', "$__workdir/$__jobname", '--run', 0,
#			'--min-seqlen', 50,
#			'--blastpgp-processors', $__blastpgp_processors,
#			'--blastdata_uniref', $__uniref,
#			'--blastpgp_seg_filter', $blastpgp_seg_filter,
#			'--psic_matrix', $psic_matrix,
#			( $debug ? ( '--debug' ) : () ) );
		if ($debug){ $cmd{psic}.=' --debug'; cluck($cmd{psic})}
		my $psicret = system( $cmd{psic} );
		if( $psicret )
		{
			if( !$__tolerate_psic_failure ){ confess( "$cmd{psic} failed: ".($?>>8) ); }
			else
			{
				my @cmd = ("touch", "$__workdir/$__jobname/$__jobname.out" );
				system( @cmd ) && confess( "'@cmd' failed: ".($?>>8));
			}
		}
	}

	#get the SIFT prediction
	if (!(-e "$__workdir/$__jobname/$__jobname.SIFTprediction")){
		#get the SIFT prediction for all mutants possible in the sequence not just ones we are interested in now
                #this allows for less re-extraction if more mutants are needed
		{
			my @cmd = ("$snapfun_utildir/makeMuts.pl", "$__workdir/$__jobname/${__jobname}_query.fasta", "all", "1");
			system( @cmd ) && confess( "'@cmd' failed: ".($?>>8) );
		}
		{	
			#before running sift in predictprotein or standalone mode we need to convert the clustal alignment into an ungapped format
			if ($mode ne "published"){
					if (-e "$__workdir/$__jobname/$__jobname.clustal"){
						cluck($cmd{formatclustal}) if $debug;
						system ($cmd{formatclustal}) && die ( "'$cmd{formatclustal}' failed: ".($?>>8) );
					}
					else { #if for any reason we're missing the alignment file - have sift produce it
						$cmd{sift}=("sift_for_submitting_fasta_seq.csh '$__workdir/$__jobname/$__jobname\_query.fasta' '$__uniref90' '$__workdir/$__jobname/$__jobname.Allmuts' '$__blastpgp_processors' ".( $debug ? '' : ' >/dev/null 2>&1' ));
					}
			}
			
			# lkajan: sift writes temporary files into the current dir :|
			if($debug){ cluck("($CWD) chdir $__workdir/$__jobname"); }
			local $CWD = "$__workdir/$__jobname";
			#my $cmd = ("sift_for_submitting_fasta_seq.csh '$__workdir/$__jobname/$__jobname\_query.fasta' '$__uniref90' '$__workdir/$__jobname/$__jobname.Allmuts' '$__blastpgp_processors' ".( $debug ? '' : ' >/dev/null 2>&1' ));

			cluck("($CWD) $cmd{sift}") if ($debug);
			if( system( $cmd{sift} ) )
			{
				if( !$__tolerate_sift_failure ){ confess("($CWD) $cmd{sift} failed: ".($?>>8)); }
				else { cluck("($CWD) $cmd{sift} failed: ".($?>>8)); } # the message here is very important even if we tolerate the failure
			}
		}
		{	
			my $siftprediction = "$__workdir/$__jobname/$__jobname\_query.SIFTprediction";
			if( !$__tolerate_sift_failure && !-e $siftprediction ){ confess("($CWD) '$siftprediction' does not exist - sift probably failed"); }
			if( -e $siftprediction )
			{
				my @cmd = ("mv", $siftprediction, "$__workdir/$__jobname/$__jobname.SIFTprediction");
				system( @cmd ) && confess( "'@cmd' failed: ".($?>>8));
			}
		}
#		{
#			my $cmd = "rm $ENV{ROOT}/sift2.1/tmp/$__jobname\_*";
#			system( $cmd ) && confess( "'$cmd' failed: ".($?>>8) );
#		}
		if( !-e "$__workdir/$__jobname/$__jobname.SIFTprediction" )
		{
			if( $__tolerate_sift_failure )
			{
				my @cmd = ("touch", "$__workdir/$__jobname/$__jobname.SIFTprediction" );
				system( @cmd ) && confess( "'@cmd' failed: ".($?>>8));
			}
			else { confess("($CWD) no '$__workdir/$__jobname/$__jobname.SIFTprediction', sift probably failed"); }
		}
	}

	#get the profbval prediction
	if (!(-e "$__workdir/$__jobname/$__jobname.profbval")){
		{
			#my @cmd = ("profbval", "$__workdir/$__jobname/$__job_fasta_file", "$__workdir/$__jobname/$__jobname-fil.rdbProf", "$__workdir/$__jobname/$__jobname-fil.hssp", "$__workdir/$__jobname/$__jobname.profbval", "1", 'snapfun');
		    cluck($cmd{profbval}) if ($debug);
			system( $cmd{profbval} ) && confess( "'$cmd{profbval}' failed: ".($?>>8) );

	    	#system( "mv $__jobname\_query.profbval $__jobname/$__jobname.profbval" );
		    if (!(-e "$__workdir/$__jobname/$__jobname.profbval")){
				my $error = "Couldn't run runPROFbvalAlign.pl to get $__jobname.profbval file\n $cmd{profbval}\n";
				return $error;
		    }
		}
	}
	
	cluck ("finally:") if ($debug);;
	return 1;
}

sub extractAll{
	# lkajan: side effect: chdir $__workdir
	my ($__workdir, $__jobname, $__seq, $__muts, $debug) = @_;

	my $check = 0;
	my @seq = split(//o, $__seq );
	open (MAIN, ">$__workdir/$__jobname/$__jobname.mutant_seqs") || die "$0:Can't open $__workdir/$__jobname/$__jobname.mutant_seqs\n";
	foreach my $mut (@$__muts){
		if ($mut =~ /\w/){
			if ($mut =~ /^\s*(.+\.)*([A-Z])\s*(\d+)\s*([A-Z])\s*(.+|\n)*/){
				my $org = $2;
				my $pos = $3;
				my $sub = $4;
				my $func = $5;
				if (!$func){
					$func = 0;
				}
				my @temp = @seq;
				if ($temp[$pos-1] eq $org){
					$temp[$pos-1] = $sub;
					print MAIN ">$__workdir/$__jobname.$org$pos$sub $func\n", join(' ', @temp), "\n";
					if (!(-e "$__workdir/$__jobname/$__jobname.$org$pos$sub.rdbProf")){
						my $ext = "fasta";
						my $profdir = $__workdir;
						open (OUT, ">$profdir/$__jobname/$__jobname.$org$pos$sub.$ext") || die "Can't open $profdir/$__jobname/$__jobname.$org$pos$sub.fasta\n";
						my $temp = join( ' ', @temp);
						$temp =~ s/\s+//g;
						$temp =~ s/([A-Z]{60})/$1\n/g;
						print OUT ">$__jobname.$org$pos$sub\n$temp";
						close OUT;
						
						{
							if( $debug ) { cluck("($CWD) chdir $profdir"); }
							local $CWD = $profdir;

							my @cmd = ("prof", "$profdir/$__jobname/$__jobname.$org$pos$sub.fasta", "dirOut=$profdir/$__jobname" ); #if( $debug ){ push @cmd, 'debug'; }
			    			cluck("@cmd") if ($debug);
							system( @cmd ) && confess("($CWD) '@cmd' failed: ".($?>>8));

							if (!(-e "$profdir/$__jobname/$__jobname.$org$pos$sub.rdbProf")){
								my $error = "Couldn't run prof to get $__jobname.$org$pos$sub.rdbProf (@cmd)";
								return $error;
							}
							unlink( "$__workdir/$__jobname/$__jobname.$org$pos$sub.fasta" );
						}
					}

					$check = 1;
				}
				else{
					my $res = $temp[$pos-1];
					my $error = "Residue at pos $pos is $res, not $org (mutant $org$pos$sub)\n";
					return $error;
				}
			}
			else{
				my $error = "Mutant $mut is of wrong format: name = $__jobname seq = $__seq mu = @$__muts";
				return $error;
			}
		}
	}
	close MAIN;

	if (!$check){
	    my $error = "No mutations found in given mutation file\n";
	    return $error;
	}
	else{
		return 1;
	}
}
1;


# vim:ai:ts=4:
