#!/usr/bin/perl
#*
#SYNOPSIS: perl ungap_clustal file
#DESCRIPTION: Converts ClustalW output in-place to alignedfasta format (SIFT readable)
#AUTHOR: M. Hecht (hecht@rostlab.org)
#*

use strict;

my $clustal=@ARGV[0];
my $outfile=$clustal;
open (FHIN,$clustal) || die "Error: unable to open $clustal \n";
my @messy=<FHIN>;
close FHIN;
open (FHOUT,"> $outfile")||die"Error: could not write $outfile\n";
my %seqhash;
my $query;
my @ungapped_output;
my $clustalheader=shift(@messy);
foreach my $line (@messy){
	next if $line=~/^\s/o;
	chomp($line);
	my ($name,$seq)=split(/\s+/o,$line); 
#	print ("$name $seq\n");
	if (!($name=~/^name\d+/o)){$query=$name}
	if ($seqhash{$name}){
		$seqhash{$name}.=$seq;
	}
	else {
		$seqhash{$name}=$seq;
	}
}
my $gapped_query_seq=$seqhash{$query};
my @gapped_query_array=split(//o,$gapped_query_seq);
my @gapindices;
for (my $i=0;$i<scalar(@gapped_query_array);$i++){
	if ($gapped_query_array[$i] eq "-") {
		push(@gapindices,$i);
#		print "$i\n";
	}
}
while( my ($name, $sequence) = each %seqhash ) {
	my $output=">$name\n";
	my @tmpseq=split(//o,$sequence);
	foreach my $index (@gapindices){
		$tmpseq[$index]="";
	}
	my $outseq=join("",@tmpseq);
	$outseq=~s/(.{60})/$1\n/g;
	$output.="$outseq\n";
	if ($name eq $query) {
		unshift(@ungapped_output,$output);
	}
	else {
		push(@ungapped_output,$output);
	}
}
foreach (@ungapped_output){
	print FHOUT "$_\n";
}
close FHOUT;
